? python_exceptions_vtk5.0.diff
? update.log
? Examples/Build/vtkMyBin
? Examples/GUI/Python/blaat.py
? Examples/Rendering/Python/DataVisDemoText.py
? Wrapping/vtkWrapPython.c.diff-20060802
? Wrapping/Python/vtk/wx/blaat.py
Index: Common/vtkPythonUtil.cxx
===================================================================
RCS file: /cvsroot/VTK/VTK/Common/vtkPythonUtil.cxx,v
retrieving revision 1.76
diff -u -r1.76 vtkPythonUtil.cxx
--- Common/vtkPythonUtil.cxx	4 Aug 2005 16:19:14 -0000	1.76
+++ Common/vtkPythonUtil.cxx	2 Aug 2006 15:41:15 -0000
@@ -403,6 +403,49 @@
 }
 
 //--------------------------------------------------------------------
+// see round about line 568 for an explanation of this event handler
+class vtkPythonErrorCommand : public vtkCommand
+{
+public:
+  static vtkPythonErrorCommand *New() {
+    return new vtkPythonErrorCommand(); };
+
+  void Execute(vtkObject *caller, unsigned long, void *);
+
+private:
+  vtkPythonErrorCommand() : vtkCommand() {};
+};
+
+void vtkPythonErrorCommand::Execute(vtkObject *vtkNotUsed(caller),
+				    unsigned long vtkNotUsed(id),
+				    void *call_data)
+{
+#if (PY_MAJOR_VERSION > 2) || \
+((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
+  PyGILState_STATE state = PyGILState_Ensure();
+#endif
+
+  // if no other Python exception has been set, we set one this means
+  // that if a Python-originating VTK method call results in a number
+  // of errors, only the first one will be stored and returned as a
+  // Python exception.
+  if (!PyErr_Occurred())
+    {
+    if (call_data)
+      PyErr_SetString(PyExc_RuntimeError, (char *)call_data);
+    else
+      PyErr_SetNone(PyExc_RuntimeError);
+    }
+
+#if (PY_MAJOR_VERSION > 2) || \
+((PY_MAJOR_VERSION == 2) && (PY_MINOR_VERSION >= 3))
+  PyGILState_Release(state);
+#endif
+}
+
+
+
+//--------------------------------------------------------------------
 static void PyVTKObject_PyDelete(PyVTKObject *self)
 {
 #if PY_VERSION_HEX >= 0x02010000
@@ -521,6 +564,31 @@
   ptr->AddObserver(vtkCommand::DeleteEvent,
                    vtkPythonDeleteCommand::New(self));
   */
+
+  // this will make sure that all objects instantiated from Python
+  // get vtkPythonErrorCommand as ErrorEvent handler.  This event
+  // handler sets up Python exception information when an object
+  // reports an error through one of the vtk error macros;
+  // this means that VTK errors can be trapped in Python as
+  // exceptions, a more natural and convenient method.
+  //
+  // Unfortunately, vtkExecutives automatically created by
+  // vtkAlgorithms cannot yet be instrumented in the same way.  One
+  // can get around this by creating a vtkOutputWindow child class
+  // that turns outputs into ErrorEvents, thus thunking back into
+  // the vtkPythonErrorCommand code.  In short:
+  // vtkAlgorithm -creates-> vtkExecutive -errors-> vtkNewOutputWindow
+  // -ErrorEvent-> vtkPythonErrorCommand -> Python RuntimeError
+  //
+  // Charl P. Botha <http://cpbotha.net/>
+  
+  if (ptr->IsA("vtkObject"))
+    {
+    ((vtkObject*)ptr)->AddObserver(vtkCommand::ErrorEvent,
+                                   vtkPythonErrorCommand::New());
+
+    } // if (ptr->IsA("vtkObject"))
+    
   return (PyObject *)self;
 }
 
Index: Wrapping/vtkWrapPython.c
===================================================================
RCS file: /cvsroot/VTK/VTK/Wrapping/vtkWrapPython.c,v
retrieving revision 1.78
diff -u -r1.78 vtkWrapPython.c
--- Wrapping/vtkWrapPython.c	20 Jul 2005 19:10:10 -0000	1.78
+++ Wrapping/vtkWrapPython.c	2 Aug 2006 15:41:17 -0000
@@ -1018,6 +1018,10 @@
                 }
               }
             fprintf(fp,");\n");
+
+
+	    fprintf(fp, "      if (PyErr_Occurred())\n");
+	    fprintf(fp, "        return NULL;\n");
           
             if (currentFunction->NumberOfArguments == 1 
                 && currentFunction->ArgTypes[0] == 0x5000)
